window.WhatsApp ?= {}
window.WhatsApp.ChatWindow = class ChatWindow extends EventEmitter
  constructor: (@app, @from, @fromUsername) ->
    @window = $('<div>').addClass('window').attr('data-id': @from).appendTo $('.chat')
    @headline = $('<div>').addClass('headline').appendTo(@window)

    @headlineName = $('<span>').addClass('name').appendTo(@headline)

    if @from.match /g\.us/i
      @headlineName.text @from
    else
      @headlineName.text (@fromUsername || @from)

    @messagesList = $('<ul>').addClass('messages').appendTo @window

    @messageInputWrapper = $('<div>').addClass('input-wrapper').appendTo @window
    @messageInput = $('<input>').addClass('message').appendTo @messageInputWrapper

    @resizeDragger = $('<div>').addClass('dragger').appendTo @window

    @closeButton = $('<div>').addClass('close').appendTo @headline

    @handleInput()
    @handleDrag()
    @handleResize()
    @handleClose()

  receiveMessage: (message) ->
    if message.from.match /g\.us/i
      senderLi = $('<li>').addClass('sender').text message.fromUsername
      senderLi.appendTo @messagesList
      
    messageLi = $('<li>').appendTo @messagesList
    messageDiv = $('<div>').addClass('received').appendTo messageLi

    t = parseInt(message.timestamp) * 1000
    time = new Date(t)
    timestamp = _.str.pad(time.getHours(), 2, "0") + ':' + _.str.pad(time.getMinutes(), 2, "0")

    if message.message?
      messageDiv.text message.message

      messageDiv.append $('<span>').addClass('time').text timestamp
      # messageDiv.emoji(16)
    if message.media?
      link = $('<a>').attr href: message.media.url, target: '_blank'
      img = $('<img>').attr(src: 'data:' + message.media.thumbType + ';base64,' + message.media.thumb).addClass('preview').appendTo link

      link.appendTo messageDiv

    @messagesList.scrollTop @messagesList[0].scrollHeight - @messagesList.height()

    unless message.from.match /g\.us/i
      if message.fromUsername?
        @headlineName.text message.fromUsername

  sentMessage: (message) ->
    messageLi = $('<li>').appendTo @messagesList
    messageDiv = $('<div>').addClass('sent').text(message.message).appendTo messageLi

    time = new Date()
    timestamp = _.str.pad(time.getHours(), 2, "0") + ':' + _.str.pad(time.getMinutes(), 2, "0")

    messageDiv.append $('<span>').addClass('time').text timestamp

    @messagesList.scrollTop @messagesList[0].scrollHeight - @messagesList.height()

  handleInput: ->
    @messageInput.keydown (e) =>
      if e.keyCode is 13
        message = @messageInput.val()
        @messageInput.val ''

        if _.str.trim(message).length > 0
          @app.sendMessage @from, message
          @sentMessage message: message

  handleResize: ->
    @resizeDragger.mousedown (e) =>
      e.preventDefault()

      lastX = e.pageX
      lastY = e.pageY

      $(document).mousemove (e) =>
        diffX = e.pageX - lastX
        diffY = e.pageY - lastY

        if @window.width() + diffX > 100
          @window.css
            width: @window.width() + diffX

          @messageInput.css
            width: @messageInput.width() + diffX

        if @window.height() + diffY > 100
          @window.css
            height: @window.height() + diffY
          @messagesList.css
            height: @messagesList.height() + diffY

        lastX = e.pageX
        lastY = e.pageY

        $(document).mouseup (e) =>
          $(document).off 'mousemove'

  handleDrag: ->
    @headline.mousedown (e) =>
      e.preventDefault()

      @bringToFront()

      lastX = e.pageX
      lastY = e.pageY

      $(document).mousemove (e) =>
        diffX = e.pageX - lastX
        diffY = e.pageY - lastY

        @window.css
          left: parseInt(@window.css('left')) + diffX
          top: parseInt(@window.css('top')) + diffY

        lastX = e.pageX
        lastY = e.pageY

        $(document).mouseup (e) =>
          $(document).off 'mousemove'

  handleClose: ->
    @closeButton.mousedown (e) ->
      e.stopPropagation()

    @closeButton.click =>
      @window.fadeOut 'fast', =>
        @emit 'close'

  show: ->
    @window.fadeIn 'fast'
    @messageInput.focus()

  bringToFront: ->
    maxZ = 0
    for el in $('.chat > *')
      if $(el).css('z-index') and $(el).css('z-index') isnt 'auto'
        maxZ = Math.max(parseInt($(el).css('z-index')), maxZ)

    newZ = maxZ + 1
    @window.css zIndex: newZ