after = (t, f) -> setTimeout f, t

decoder  = require './decode'
protocol = require './protocol'
func     = require './func'
unpack   = require './unpack'
rc4      = require './rc4'
request  = require 'request'
ord      = require './ord'

_        = require './underscore'
_.str    = require './underscore.string'

net            = require 'net'
{EventEmitter} = require 'events'
crypto         = require 'crypto'

class global.IncompleteMessageException
  constructor: (@message) ->
    @name = 'Incomplete Exception'

  setInput: (@input) ->
  getInput: -> @input

module.exports = class WhatsProt extends EventEmitter
  disconnectedStatus: 'disconnected'
  connectedStatus: 'connected'

  whatsAppHost: 'bin-short.whatsapp.net'
  whatsAppServer: 's.whatsapp.net'
  whatsAppGroupHost: 'g.us'
  whatsAppRealm: 's.whatsapp.net'
  whatsAppDigest: 'xmpp/s.whatsapp.net'
  device: 'iPhone'
  whatsAppVer: '2.8.4'
  port: 5222
  timeout: { sec: 2, usec: 0 }
  constructor: (@phoneNumber, @imei, @name, @debug = false) ->
    @messageQueue = []
    @incomplete_message = null

    dict = decoder.getDictionary()
    @writer = new protocol.BinTreeNodeWriter dict
    @reader = new protocol.BinTreeNodeReader dict
    @loginStatus = @disconnectedStatus

    @rc4engine = new rc4.Engine()

  addFeatures: ->
    child = new protocol.ProtocolNode 'receipt_acks', null, null, ''
    child_groups = new protocol.ProtocolNode 'groups', null, null, ''
    parent = new protocol.ProtocolNode 'stream:features', null, [child, child_groups], ''
    return parent

  addAuth: ->
    authHash = {}
    authHash.xmlns = 'urn:ietf:params:xml:ns:xmpp-sasl'
    authHash.mechanism = 'WAUTH-1'
    authHash.user = @phoneNumber
    node = new protocol.ProtocolNode 'auth', authHash, null, ''
    return node

  encryptPassword: ->
    if ~@imei.indexOf ':'
      @imei = @imei.toUpperCase()
      return @md5(@imei + @imei)
    else
      return @md5(@imei.split('').reverse().join(''))

  authenticate: (challenge, callback) ->
    # Encrypt password and challenge using pbkdf2
    crypto.pbkdf2 @encryptPassword(), challenge, 16, challenge.length, (ex, buf) =>
      buffer = []
      for char in @phoneNumber
        buffer.push ord(char)
      
      for char in challenge
        buffer.push char

      for char in Math.round(new Date() / 1000).toString()
        buffer.push ord(char)

      # Encrypt the buffer using RC4 (use pbkdf2 encoded password as salt?)
      @rc4engine.init buf
      @rc4engine.drop 256

      buffer = @rc4engine.encrypt buffer

      hash = crypto.createHmac('sha1', buf).update(buffer).digest('binary')

      hashPart = [ord(hash[0]), ord(hash[1]), ord(hash[2]), ord(hash[3])]

      bufferArr = []
      for char in buffer
        bufferArr.push char

      callback hashPart.concat(bufferArr)

  ###
    OLD AUTHENTICATION METHOD (Protocol 1.1)
  ###
  # authenticate: (nonce) ->
  #   NC = '00000001'
  #   qop = 'auth'

  #   cnonce = func.random_uuid()

  #   data1 = @phoneNumber
  #   data1 += ':'
  #   data1 += @whatsAppServer
  #   data1 += ':'
  #   data1 += @encryptPassword()

  #   data2 = unpack.pack('H32', @md5(data1))
  #   data2 += ':'
  #   data2 += nonce
  #   data2 += ':'
  #   data2 += cnonce

  #   data3 = 'AUTHENTICATE:'
  #   data3 += @whatsAppDigest

  #   data4 = @md5(data2)
  #   data4 += ':'
  #   data4 += nonce
  #   data4 += ':'
  #   data4 += NC
  #   data4 += ':'
  #   data4 += cnonce
  #   data4 += ':'
  #   data4 += qop
  #   data4 += ':'
  #   data4 += @md5(data3)

  #   data5 = @md5(data4)

  #   response = _.str.sprintf('username="%s",realm="%s",nonce="%s",cnonce="%s",nc=%s,qop=%s,digest-uri="%s",response=%s,charset=utf-8',
  #     @phoneNumber,
  #     @whatsAppRealm,
  #     nonce,
  #     cnonce,
  #     NC,
  #     qop,
  #     @whatsAppDigest,
  #     data5)

  #   return response

  generateAuthResponse: (callback) ->
    @authenticate @challenge, (resp) =>
      console.log "response is ", new Buffer(resp), resp.length

      respHash = {}
      respHash.xmlns = 'urn:ietf:params:xml:ns:xmpp-sasl'
      node = new protocol.ProtocolNode 'response', respHash, null, resp
      callback node

  sendData: (data) ->
    @socket.write new Buffer(data), 'binary' #TODO

    console.log "sending data", new Buffer(data)

  sendNode: (node) ->
    @debugPrint node.NodeString('tx  ') + '\n'
    data = @writer.write(node)
    @sendData data

  processChallenge: (node) ->
    @challenge = new Buffer(node.data)

  message: (msgid, to, txt, group=false) ->
    bodyNode = new protocol.ProtocolNode "body", null, null, txt
    @sendMessageNode msgid, to, bodyNode, group

  messageImage: (msgid, to, url, file, size, icon) ->
    mediaAttribs = {}
    mediaAttribs.xmlns = 'urn:xmpp:whatsapp:mms'
    mediaAttribs.type = 'image'
    mediaAttribs.url = url
    mediaAttribs.file = file
    mediaAttribs.size = size

    mediaNode = new protocol.ProtocolNode 'media', mediaAttribs, null, icon
    @sendMessageNode msgid, to, mediaNode, false

  sendMessageNode: (msgid, to, node, group=false) ->
    serverNode = new protocol.ProtocolNode "server", null, null, ""

    xHash = {}
    xHash.xmlns = 'jabber:x:event'
    xNode = new protocol.ProtocolNode "x", xHash, [serverNode], ""

    messageHash = {}
    if group
      messageHash.to = to + "@" + @whatsAppGroupHost
    else
      messageHash.to = to + "@" + @whatsAppServer
    messageHash.type = 'chat'
    messageHash.id = msgid

    messageNode = new protocol.ProtocolNode 'message', messageHash, [xNode, node], ""
    @sendNode messageNode

  sendMessageReceived: (msg) ->
    requestNode = msg.getChild 'request'
    if requestNode isnt null
      xmlnsAttrib = requestNode.getAttribute('xmlns')
      if xmlnsAttrib is 'urn:xmpp:receipts'
        receivedHash = {}
        receivedHash.xmlns = 'urn:xmpp:receipts'

        receivedNode = new protocol.ProtocolNode 'received', receivedHash, null, ''

        messageHash = {}
        messageHash.to = msg.getAttribute 'from'
        messageHash.type = 'chat'
        messageHash.id = msg.getAttribute 'id'
        messageNode = new protocol.ProtocolNode 'message', messageHash, [receivedNode], ''

        @sendNode messageNode

  processInboundData: (data) ->
    try
      node = @reader.nextTree data
      while node isnt null
        @debugPrint node.NodeString('rx  ') + '\n'
        if node.tag is 'start'
          
        else if node.tag is 'challenge'
          @processChallenge node
          
          @generateAuthResponse (authNode) =>
            @sendNode authNode
        else if node.tag is 'success'
          @loginStatus = @connectedStatus
          @accountInfo = {
            status: node.getAttribute('status'),
            kind:   node.getAttribute('kind'),
            creation: node.getAttribute('creation'),
            expiration: node.getAttribute('expiration')
          }
          @emit 'auth_response', true
        else if node.tag is 'failure'
          @emit 'auth_response', false, node.children[0].tag
        else if node.tag is 'stream:error'
          @emit 'error', _.find(node.children, ((child) -> child.tag is 'text')).data.toString()
        if node.tag is 'message'
          body = _.find(node.children, ((child) -> child.tag is 'body'))
          media = _.find(node.children, ((child) -> child.tag is 'media'))
          if body? or media?
            fromUsername = _.find(node.children, ((child) -> child.tag is 'notify'))?.attributeHash.name.toString()

            if fromUsername.match /[0-9]+/i
              fromUsername = "+" + fromUsername

            message = 
              from: node.attributeHash.from.toString()
              fromUsername: fromUsername
              timestamp: node.attributeHash.t.toString()

            if body?
              message.message = body?.data.toString()
            if media?
              message.media =
                thumb: media?.data.toString()
                thumbType: media?.attributeHash.mimetype?.toString() || 'image/jpeg'
                url: media?.attributeHash.url.toString()

            @emit 'wa_message', message

          @sendMessageReceived node
        if node.tag is 'iq' and node.attributeHash.type is "get" and node.children[0].tag is 'ping'
          @pong node.attributeHash['id']

        node = @reader.nextTree()
    catch e
      console.log e
      @incomplete_message = e.getInput()

  sendNickname: (nickname) ->
    messageHash = {}
    messageHash.name = nickname
    messageNode = new protocol.ProtocolNode 'presence', messageHash, null, ''
    @sendNode messageNode

  close: ->
    @socket.end()

  connect: ->
    @socket = net.connect host: @whatsAppHost, port: @port
    @socket.setKeepAlive true
    @socket.on 'connect', =>
      @emit 'connected'

    @socket.on 'data', (data) =>
      if @incomplete_message?
        buff = Buffer.concat([@incomplete_message, data])
      else
        buff = data

      @incomplete_message = null

      @processInboundData buff

    @socket.on 'error', (e) =>
      console.log e

    @socket.on 'timeout', (e) =>
      console.log "timeout"

    @socket.on 'close', =>
      console.log "disconnected"
      @emit 'disconnect'

  login: (callback) ->
    resource = "#{@device}-#{@whatsAppVer}-#{@port}"
    data = @writer.startStream @whatsAppServer, resource
    feat = @addFeatures()
    @sendData data
    @sendNode feat
    @sendNode @addAuth()

    @on 'auth_response', (success, err) =>
      @sendNickname @name
      callback? success, err


  pong: (msgid) ->
    whatsAppServer = @whatsAppServer

    messageHash = {}
    messageHash.to = whatsAppServer
    messageHash.id = msgid
    messageHash.type = 'result'

    messageNode = new protocol.ProtocolNode 'iq', messageHash, null, ''

    @sendNode messageNode

  debugPrint: (message) ->
    console.log(message) if @debug

  md5: (input) ->
    return require('crypto').createHash('md5').update(input).digest('hex')
