(function() {
  var ChatWindow, _ref,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  if ((_ref = window.WhatsApp) == null) {
    window.WhatsApp = {};
  }

  window.WhatsApp.ChatWindow = ChatWindow = (function(_super) {

    __extends(ChatWindow, _super);

    function ChatWindow(app, from, fromUsername) {
      this.app = app;
      this.from = from;
      this.fromUsername = fromUsername;
      this.window = $('<div>').addClass('window').attr({
        'data-id': this.from
      }).appendTo($('.chat'));
      this.headline = $('<div>').addClass('headline').appendTo(this.window);
      this.headlineName = $('<span>').addClass('name').appendTo(this.headline);
      if (this.from.match(/g\.us/i)) {
        this.headlineName.text(this.from);
      } else {
        this.headlineName.text(this.fromUsername || this.from);
      }
      this.messagesList = $('<ul>').addClass('messages').appendTo(this.window);
      this.messageInputWrapper = $('<div>').addClass('input-wrapper').appendTo(this.window);
      this.messageInput = $('<input>').addClass('message').appendTo(this.messageInputWrapper);
      this.resizeDragger = $('<div>').addClass('dragger').appendTo(this.window);
      this.closeButton = $('<div>').addClass('close').appendTo(this.headline);
      this.handleInput();
      this.handleDrag();
      this.handleResize();
      this.handleClose();
    }

    ChatWindow.prototype.receiveMessage = function(message) {
      var img, link, messageDiv, messageLi, senderLi, t, time, timestamp;
      if (message.from.match(/g\.us/i)) {
        senderLi = $('<li>').addClass('sender').text(message.fromUsername);
        senderLi.appendTo(this.messagesList);
      }
      messageLi = $('<li>').appendTo(this.messagesList);
      messageDiv = $('<div>').addClass('received').appendTo(messageLi);
      t = parseInt(message.timestamp) * 1000;
      time = new Date(t);
      timestamp = _.str.pad(time.getHours(), 2, "0") + ':' + _.str.pad(time.getMinutes(), 2, "0");
      if (message.message != null) {
        messageDiv.text(message.message);
        messageDiv.append($('<span>').addClass('time').text(timestamp));
      }
      if (message.media != null) {
        link = $('<a>').attr({
          href: message.media.url,
          target: '_blank'
        });
        img = $('<img>').attr({
          src: 'data:' + message.media.thumbType + ';base64,' + message.media.thumb
        }).addClass('preview').appendTo(link);
        link.appendTo(messageDiv);
      }
      this.messagesList.scrollTop(this.messagesList[0].scrollHeight - this.messagesList.height());
      if (!message.from.match(/g\.us/i)) {
        if (message.fromUsername != null) {
          return this.headlineName.text(message.fromUsername);
        }
      }
    };

    ChatWindow.prototype.sentMessage = function(message) {
      var messageDiv, messageLi, time, timestamp;
      messageLi = $('<li>').appendTo(this.messagesList);
      messageDiv = $('<div>').addClass('sent').text(message.message).appendTo(messageLi);
      time = new Date();
      timestamp = _.str.pad(time.getHours(), 2, "0") + ':' + _.str.pad(time.getMinutes(), 2, "0");
      messageDiv.append($('<span>').addClass('time').text(timestamp));
      return this.messagesList.scrollTop(this.messagesList[0].scrollHeight - this.messagesList.height());
    };

    ChatWindow.prototype.handleInput = function() {
      var _this = this;
      return this.messageInput.keydown(function(e) {
        var message;
        if (e.keyCode === 13) {
          message = _this.messageInput.val();
          _this.messageInput.val('');
          if (_.str.trim(message).length > 0) {
            _this.app.sendMessage(_this.from, message);
            return _this.sentMessage({
              message: message
            });
          }
        }
      });
    };

    ChatWindow.prototype.handleResize = function() {
      var _this = this;
      return this.resizeDragger.mousedown(function(e) {
        var lastX, lastY;
        e.preventDefault();
        lastX = e.pageX;
        lastY = e.pageY;
        return $(document).mousemove(function(e) {
          var diffX, diffY;
          diffX = e.pageX - lastX;
          diffY = e.pageY - lastY;
          if (_this.window.width() + diffX > 100) {
            _this.window.css({
              width: _this.window.width() + diffX
            });
            _this.messageInput.css({
              width: _this.messageInput.width() + diffX
            });
          }
          if (_this.window.height() + diffY > 100) {
            _this.window.css({
              height: _this.window.height() + diffY
            });
            _this.messagesList.css({
              height: _this.messagesList.height() + diffY
            });
          }
          lastX = e.pageX;
          lastY = e.pageY;
          return $(document).mouseup(function(e) {
            return $(document).off('mousemove');
          });
        });
      });
    };

    ChatWindow.prototype.handleDrag = function() {
      var _this = this;
      return this.headline.mousedown(function(e) {
        var lastX, lastY;
        e.preventDefault();
        _this.bringToFront();
        lastX = e.pageX;
        lastY = e.pageY;
        return $(document).mousemove(function(e) {
          var diffX, diffY;
          diffX = e.pageX - lastX;
          diffY = e.pageY - lastY;
          _this.window.css({
            left: parseInt(_this.window.css('left')) + diffX,
            top: parseInt(_this.window.css('top')) + diffY
          });
          lastX = e.pageX;
          lastY = e.pageY;
          return $(document).mouseup(function(e) {
            return $(document).off('mousemove');
          });
        });
      });
    };

    ChatWindow.prototype.handleClose = function() {
      var _this = this;
      this.closeButton.mousedown(function(e) {
        return e.stopPropagation();
      });
      return this.closeButton.click(function() {
        return _this.window.fadeOut('fast', function() {
          return _this.emit('close');
        });
      });
    };

    ChatWindow.prototype.show = function() {
      this.window.fadeIn('fast');
      return this.messageInput.focus();
    };

    ChatWindow.prototype.bringToFront = function() {
      var el, maxZ, newZ, _i, _len, _ref1;
      maxZ = 0;
      _ref1 = $('.chat > *');
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        el = _ref1[_i];
        if ($(el).css('z-index') && $(el).css('z-index') !== 'auto') {
          maxZ = Math.max(parseInt($(el).css('z-index')), maxZ);
        }
      }
      newZ = maxZ + 1;
      return this.window.css({
        zIndex: newZ
      });
    };

    return ChatWindow;

  })(EventEmitter);

}).call(this);
